#include "ALSEngine.h"

ALSEngine::ALSEngine(
  Matrix ratingMatrix,
  double alpha,
  double regularization,
  size_t factors,
  size_t iterations,
  size_t seed
):  ratingMatrix(ratingMatrix),
    alpha(alpha),
    regularization(regularization),
    factors(factors),
    iterations(iterations),
    seed(seed)
{
  this->ratingMatrix = this->ratingMatrix * this->alpha;
  this->numUsers = this->ratingMatrix.rows();
  this->numItems = this->ratingMatrix.cols();

  // set seed for random
  std::srand(this->seed);

  this->userFactors = Matrix::random(this->numUsers, this->factors);
  this->itemFactors = Matrix::random(this->numItems, this->factors);
}

ALSEngine::ALSEngine(
  Matrix ratingMatrix,
  Matrix userFactors,
  Matrix itemFactors,
  double regularization,
  size_t iterations
):  ratingMatrix(ratingMatrix),
    userFactors(userFactors),
    itemFactors(itemFactors),
    regularization(regularization),
    iterations(iterations)
{
}

void ALSEngine::solver(Matrix& Cui, Matrix& X, Matrix& Y, double regularization){
  size_t users = X.rows();
  size_t factors = X.cols();
  Matrix YtY = Y.transpose() * Y;

  for(size_t u = 0; u < users; u++) {
    Matrix A = YtY + ( Matrix::identity(factors) * regularization );
    Vector b(factors);

    for(size_t i = 0; i < Cui.cols(); i++ ){
      double confidence = Cui(u,i);
      if(confidence == 0 ) continue;

      Vector factor = Y(i);
      A = A + ( Matrix::outer(factor, factor) * (confidence - 1) );
      b = b + ( factor * confidence );
    }

    X.set(u, Matrix::solve(A, b));
  }
}

Matrix ALSEngine::train(){
  Matrix tRatingMatrix = this->ratingMatrix.transpose();

  for(size_t i = 0; i < iterations; i++){
    solver(this->ratingMatrix, this->userFactors, this->itemFactors, this->regularization);
    solver(tRatingMatrix, this->itemFactors, this->userFactors, this->regularization);
  }

  Matrix X = this->userFactors * this->itemFactors.transpose();

  return X;
}
