#pragma once

#include <random>
#include <algorithm>
#include <vector>
#include <unordered_set>
#include <stdlib.h>
#include "Matrix.h"
#include "Vector.h"
#include "IdIndex.h"

class BPREngine {
  public:
    BPREngine(
      Matrix ratingMatrix,
      size_t factors,
      size_t iterations,
      size_t seed,
      double userLambda,
      double itemLambda,
      double learningRate,
      size_t numNegativeSamples,
      size_t evalNumNeg,
      int32_t evalSeed,
      double decayRate,
      bool shuffleTrainingSet,
      double initDistributionBound
    );

    Matrix optimize();

  private:
    struct PosPair {
      size_t userIdx;
      size_t posItemIdx;
    };

    struct PosNegTriplet {
      size_t userIdx;
      size_t posItemIdx;
      size_t negItemIdx;
    };

    double predictDifference(const size_t userIdx, const size_t posItemIdx, const size_t negItemIdx) const;
    double loss(const double scoreDifference) const;
    double lossDerivative(const double scoreDifference) const;

    void evaluate();
    void update(const PosNegTriplet& triplet);

    void shuffle();

    template <typename FuncT, typename GenT>
    void iterate(FuncT func, const size_t numNeg, GenT&& gen) const;

    template <typename GenT>
    size_t sampleRandomNegative(const size_t userIdx, GenT&& gen, const bool useTestItemMap = false) const;

    std::vector<PosPair> data_;
    std::vector<PosNegTriplet> evalSet_;

    Matrix ratingMatrix;
    size_t numUsers;
    size_t numItems;

    IdIndex userIndex_;
    IdIndex itemIndex_;

    using ItemMap = std::vector<std::unordered_set<size_t>>;
    ItemMap itemMap_;

    size_t factors;
    size_t iterations;

    double _learningRate;
    double userLambda;
    double itemLambda;

    double decayRate;
    bool shuffleTrainingSet;

    const size_t numNegativeSamples;
    const size_t evalNumNeg_;
    const int32_t evalSeed_;

    double initDistributionBound;

    std::mt19937 gen_;

    Matrix userFactors;
    Matrix itemFactors;

};
