#pragma once

#include <limits>
#include <vector>
#include <unordered_map>


class IdIndex {
 public:
  static const size_t missingIdx = std::numeric_limits<size_t>::max();

  IdIndex() = default;

  int64_t id(const size_t idx) const {
    return ids_[idx];
  }

  size_t idx(const int64_t id) const {
    const auto pos = idxMap_.find(id);
    return (pos != idxMap_.end() ? pos->second : missingIdx);
  }

  size_t getOrSetIdx(const int64_t id);

  size_t size() const {
    return ids_.size();
  }

  const std::vector<int64_t>& ids() const {
    return ids_;
  }

 private:
  std::vector<int64_t> ids_;
  std::unordered_map<int64_t, size_t> idxMap_;
};
