#pragma once

#include <stdlib.h>
#include "Matrix.h"
#include "Vector.h"

class LogisticMFEngine {
  public:
    LogisticMFEngine(
      Matrix ratingMatrix,
      size_t factors, size_t iterations,
      double regularization, double gamma,
      size_t seed
    );

    LogisticMFEngine(
      Matrix ratingMatrix,
      Matrix userVectors, Matrix itemVectors,
      Vector userBiases, Vector itemBiases,
      double regularization, double gamma,
      size_t iterations, size_t factors
    );

    Matrix train();

  private:

    void derive(bool user, Matrix& vecDeriv, Matrix& biasDeriv);

    size_t numUsers;
    size_t numItems;

    Matrix ones;
    Matrix ratingMatrix;
    Matrix userVectors;
    Matrix itemVectors;
    Vector userBiases;
    Vector itemBiases;

    double regularization;
    double gamma;

    size_t factors;
    size_t iterations;
    size_t seed;

};
