
#include "Metrics.h"

#include <functional>
#include <algorithm>

double AUC::compute(const std::vector<double>& labels,
                    const std::vector<double>& scores) const {

  std::vector<std::pair<double, bool>> scored;
  scored.reserve(labels.size());
  int32_t pos = 0;
  for (size_t i = 0; i < labels.size(); ++i) {
    if (labels[i] > 0.0) {
      ++pos;
    }
    scored.emplace_back(scores[i], static_cast<bool>(labels[i] > 0.0));
  }
  int32_t neg = labels.size() - pos;

  // make sure we have at least 1 example in each class
  if (pos == 0 || neg == 0) {
    return 1.0;
  }

  std::sort(
    scored.begin(), scored.end(), std::greater<std::pair<double, bool>>());
  int tp = 0;
  double auc = 0;
  for (const auto& p : scored) {
    if (p.second) {
      ++tp;
    } else {
      // tpr * d(fpr) = (tp/pos) * (1/neg)
      auc += static_cast<double>(tp) / pos / neg;
    }
  }

  return auc;
}
